/* stats.h

   Collect data for mean, max, min, &c

   Oww project
   Dr. Simon J. Melhuish

   Fri 01st December 2000
*/

#ifndef STATS_H
#define STATS_H 1
#ifndef WSTYPES_H
#  include "wstypes.h"
#endif
#ifndef __time_h
#  include <time.h>
#endif

typedef struct {
  int N ;                 /* Number of contributing readings */
  time_t timeBase ;       /* time of first contributing readings */
  time_t sumTime ;        /* sum of (time - timeBase) */
  time_t last_t ;         /* Time of the last readout before this integration */
  float sumT[MAXTEMPS] ;  /* Temperature sums */
  int   T_N[MAXTEMPS] ;   /* Temperature N */
  float sumWx ;           /* Sum of wind x components */
  float sumWy ;           /* Sum of wind y components */
  float sumWs ;           /* Sum of wind speeds */
  float minT[MAXTEMPS] ;  /* Minimum temperatures */
  float minWs ;           /* Minimum wind speed */
  float maxT[MAXTEMPS] ;  /* Maximum temperatures */
  float maxWs ;           /* Maximum wind speed */
  int bearing ;           /* Latest wind bearing */
  float rain ;            /* Rainfall (not monotonic) */
  int rainN ;             /* Number of rainfall readings taken */
  unsigned long rainNow ; /* Latest rain reading (monotonic) */
  unsigned long rainOld ; /* Rain reading from previous integration */
  float gpc[MAXGPC] ;            /* GPC values (not monotonic) */
  unsigned long gpcNow[MAXGPC] ; /* Latest GPC readings (monotonic) */
  unsigned long gpcOld[MAXGPC] ; /* GPC readings from previous integration */
  unsigned long gpcEvents[MAXGPC] ; /* Latest GPC event */
  float gpceventmax[MAXGPC] ; /* Maximum GPC event delta */
  int gpcN[MAXGPC] ;      /* GPC readings */
  float sumTrh[MAXHUMS] ; /* Temperature sums from RH sensors */
  float minTrh[MAXHUMS] ; /* Minimum temperatures from RH sensors */
  float maxTrh[MAXHUMS] ; /* Maximum temperatures from RH sensors */
  float  sumRH[MAXHUMS] ; /* RH sums */
  int     RH_N[MAXHUMS] ; /* RH N from RH sensors */
  float  minRH[MAXHUMS] ; /* Minimum RH from RH sensors */
  float  maxRH[MAXHUMS] ; /* Maximum RH from RH sensors */
  float  sumbarom[MAXBAROM] ; /* barom sums */
  int     barom_N[MAXBAROM] ; /* barom N from barom sensors */
  float  minbarom[MAXBAROM] ; /* Minimum barom from barom sensors */
  float  maxbarom[MAXBAROM] ; /* Maximum barom from barom sensors */
  float sumTb[MAXBAROM] ; /* Temperature sums from bp sensors */
  float minTb[MAXBAROM] ; /* Minimum temperatures from bp sensors */
  float maxTb[MAXBAROM] ; /* Maximum temperatures from bp sensors */
  int   sol_N[MAXSOL] ;   /* Solar radiation No. of values */
  float sol_sum[MAXSOL] ; /* Solar radiation summation */
  float sol_min[MAXSOL] ; /* Solar radiation minimum */
  float sol_max[MAXSOL] ; /* Solar radiation maximum */
} statsstruct ;

typedef struct {
  time_t meanTime ;       /* Mean time for this integration */
  struct tm meanTime_tm ; /* Mean time as struct tm - localtime */
  float rain ;            /* Rainfall (not monotonic) */
  float deltaRain ;       /* Rainfall this integration */
  float rain_rate ;       /* Rainfall rate */
  float gpc[MAXGPC] ;     /* Absolute gpc */
  float deltagpc[MAXGPC] ; /* GPC increment this integration */
  unsigned long gpcmono[MAXGPC] ; /* Monotonic gpc count */
  unsigned long gpcEvents[MAXGPC] ; /* Gpc event counter */
  float gpceventmax[MAXGPC] ; /* Gpc maximum event delta */
  float gpcrate[MAXGPC] ; /* gpc rate */
  int haveGPC[MAXGPC] ;   /* Valid GPC value? */
  float meanWd ;          /* Mean wind direction */
  float meanWs ;          /* Mean wind speed */
  float maxWs ;           /* Maximum wind speed */
  int point ;             /* Closest compass point */
  const char *pointName ; /* Compass point as a name */
  float meanT[MAXTEMPS] ; /* Temperature means */
  int   haveT[MAXTEMPS] ; /* Valid meanT[i] value? */
  float meanTrh[MAXHUMS] ;/* Temperature means from RH sensors */
  float meanRH[MAXHUMS] ; /* RH means from RH sensors */
  int   haveRH[MAXHUMS] ; /* Valid meanRH[i] value? */
  float meanbarom[MAXBAROM] ; /* barometer means from barometer sensors */
  float meanTb[MAXBAROM] ; /* T means from barometer sensors */
  int   havebarom[MAXBAROM] ; /* Valid meanbarom[i] value? */
  float meansol[MAXSOL] ; /* Mean solar radiation */
  int havesol[MAXSOL] ; /* Valid meansol[i] value? */
} statsmean ;

void stats_new_anem(wsstruct vals, statsstruct *stats) ;
void stats_new_gpc(wsstruct vals, statsstruct *stats) ;
void stats_new_data(wsstruct vals, statsstruct *stats) ;
void stats_reset(statsstruct *stats) ;
int  stats_do_means(statsstruct *stats, statsmean *means) ;
int  stats_do_ws_means(wsstruct *vals, statsmean *means) ;
struct tm *stats_mean_tm(statsstruct *stats) ;
time_t stats_mean_t(statsstruct *stats) ;

#endif

